<#
    Script Name: VMware vSphere Security Settings Audit Utility
    Author: Bob Plankers (rplankers@vmware.com)
    Date: 2023-09-28
    Copyright (C) 2023 VMware, Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
#>

Param (
    # Output File Name
    [Parameter(Mandatory=$true)]
    [string]$Directory
)

#######################################################################################################
# Header
$currentDateTime = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
Write-Output "[INFO] VMware vSphere Security Configuration & Hardening Guide 802-20230930-01"
Write-Output "[INFO] vSphere Security Settings Audit Utility"
Write-Output "[INFO]"

#####################
# Check to see if we have the required version of VMware.PowerCLI
$installedVersion = (Get-InstalledModule -Name 'VMware.PowerCLI' -AllVersions -ErrorAction SilentlyContinue).Version | Sort-Object -Desc | Select-Object -First 1

if ('13.0.0' -gt $installedVersion) {
    Write-Output "[ERROR] This script requires PowerCLI 13.0 or newer. Current version is $installedVersion" 
    Write-Output "[ERROR] Instructions for installation & upgrade can be found at https://developer.vmware.com/powercli" 
    Exit
}

#####################
# Check to see if we are attached to a vCenter Server
if ($global:DefaultVIServers.Count -lt 1) {
    Write-Output "[ERROR] Please connect to a single vCenter Server (use Connect-VIServer, Connect-CisServer, and Connect-SsoAdminServer) prior to running this script. Thank you."
    Exit
}

#####################
# Read the ESXi host into objects and views once to save time & resources
$vms = Get-VM | Sort-Object -Property Name
$hosts = Get-VMHost | Sort-Object -Property Name

#####################
# Check to see if everything is the right version
foreach ($hostVersion in $hosts.Version) {
    if ('8.0.0' -gt $hostVersion) {
        Write-Output "[ERROR] This script is designed for vSphere 8. Please upgrade all ESXi hosts." 
        Exit
    }

    if ('8.0.0' -gt $global:DefaultVIServers.Version) {
        Write-Output "[ERROR] This script is designed for vSphere 8. Please upgrade the connected vCenter." 
        Exit
    }
}    

foreach ($vm in $vms) {
    .\audit-vm-8.ps1 -name $vm.Name -NoBanners -NoSafetyChecksExceptAppliances | Tee-Object -Append -FilePath "$directory\$($vm.Name).txt"
}

foreach ($esxi in $hosts) {
    .\audit-esxi-8.ps1 -name $esxi -NoBanners -NoSafetyChecks | Tee-Object -Append -FilePath "$directory\$($esxi.Name).txt"
}

.\audit-vcenter-8.ps1 -NoBanners -NoSafetyChecks | Tee-Object -Append -FilePath "$directory\$($global:DefaultVIServers.Name).txt"