<#
    Script Name: VMware vSphere ESXi Host Security Settings Audit Utility
    Copyright (C) 2023 VMware, Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
#>

<#
    "And now that you don’t have to be perfect, you can be good." ~ John Steinbeck, East of Eden
#>

Param (
    # ESXi Host Name
    [Parameter(Mandatory=$true)]
    [string]$Name,
    # Omit all banners
    [Parameter(Mandatory=$false)]
    [switch]$NoBanners = $false,
    # Skip safety checks
    [Parameter(Mandatory=$false)]
    [switch]$NoSafetyChecks = $false
)

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if we have the required version of VMware.PowerCLI
    $installedVersion = (Get-InstalledModule -Name 'VMware.PowerCLI' -AllVersions -ErrorAction SilentlyContinue).Version | Sort-Object -Desc | Select-Object -First 1

    if ('13.0.0' -gt $installedVersion) {
        Write-Output "[ERROR] This script requires PowerCLI 13.0 or newer. Current version is $installedVersion" 
        Write-Output "[ERROR] Instructions for installation & upgrade can be found at https://developer.vmware.com/powercli" 
        Exit
    }

    #####################
    # Check to see if we are attached to a vCenter Server
    if ($global:DefaultVIServers.Count -lt 1) {
        Write-Output "[ERROR] Please connect to a vCenter Server (use Connect-VIServer) prior to running this script. Thank you." 
        Exit
    }
}

if ($NoBanners -eq $false) {
    
    #######################################################################################################
    # Header
    Write-Output "$name`:   [TEXT] VMware vSphere Security Configuration & Hardening Guide 802-20230930-01"
    Write-Output "$name`:   [TEXT] ESXi Host Security Settings Audit Utility"
    Write-Output "$name`:   [TEXT]"
}

# You want the date and time in the audit record.
$currentDateTime = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
Write-Output "$name`:   [INFO] Audit of $name started at $currentDateTime from $env:COMPUTERNAME by $env:USERNAME"
Write-Output "$name`:   [INFO]"

#####################
# Read the ESXi host into objects and views once to save time & resources
$obj = Get-VMHost $name -ErrorAction Stop
$view = Get-View -VIObject $obj
$ESXcli = Get-EsxCli -VMHost $obj -V2

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if the host is the right version
    if ('8.0.0' -gt $obj.Version) {
        Write-Output "$name`:  [ERROR] This script is designed for ESXi 8. Please upgrade the specified host." 
        Exit
    }
}

if ($NoBanners -eq $false) {
  
    #####################
    # Disclaimer
    Write-Output ("$name`:   [TEXT] See the included documentation for license information and disclaimer.") 
    Write-Output ("$name`:   [TEXT]")

}

#####################
# Tests for advanced parameters
$scg_adv = @{
    
    'Security.AccountUnlockTime' = 900
    'Security.AccountLockFailures' = 5
    'Security.PasswordQualityControl' = 'retry=3 min=disabled,15,15,15,15 max=64 similar=deny passphrase=3'
    'Security.PasswordHistory' = 5
    'Security.PasswordMaxDays' = 99999
    'Config.HostAgent.vmacore.soap.sessionTimeout' = 30
    'Config.HostAgent.plugins.solo.enableMob' = $false
    'UserVars.DcuiTimeOut' = 600
    'UserVars.SuppressHyperthreadWarning' = 0
    'UserVars.SuppressShellWarning' = 0
    'UserVars.HostClientSessionTimeout' = 900
    'Net.BMCNetworkEnable' = 0
    'DCUI.Access' = 'root'
    'Syslog.global.auditRecord.storageEnable' = $true
    'Syslog.global.auditRecord.storageCapacity' = 100
    'Syslog.global.auditRecord.remoteEnable' = $true
    'Syslog.global.logFiltersEnable' = $false
    'Config.HostAgent.log.level' = 'info'
    'Syslog.global.logLevel' = 'info'
    'Syslog.global.certificate.checkSSLCerts' = $true
    'Syslog.global.certificate.strictX509Compliance' = $true
    'Net.BlockGuestBPDU' = 1
    'Net.DVFilterBindIpAddress' = ''
    'UserVars.ESXiShellInteractiveTimeOut' = 900
    'UserVars.ESXiShellTimeOut' = 600
    'UserVars.ESXiVPsDisabledProtocols' = "sslv3,tlsv1,tlsv1.1"
    'Mem.ShareForceSalting' = 2
    'VMkernel.Boot.execInstalledOnly' = $true
    'Mem.MemEagerZero' = 1

}

foreach ($param in $scg_adv.GetEnumerator() )
{
    $vmval = (Get-AdvancedSetting -Entity $obj "$($param.Name)").Value

    if ($vmval -eq $($param.Value)) {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)" 
    } else {
        Write-Output "$name`:   [FAIL] $($param.Name) not configured correctly ($vmval)" 
    }
}

#####################
# Tests for things that should not be set a certain way
$scg_not = @{

    'Annotations.WelcomeMessage' = ''
    'Config.Etc.Issue' = ''
    'Syslog.global.auditRecord.storageDirectory' = '[] /scratch/auditLog'
    'Syslog.global.logDir' = '[] /scratch/log'
    'Syslog.global.logHost' = ''

}

foreach ($param in $scg_not.GetEnumerator() )
{
    $vmval = (Get-AdvancedSetting -Entity $obj "$($param.Name)").Value

    if ($vmval -eq $($param.Value)) {
        Write-Output "$name`:   [FAIL] $($param.Name) not configured correctly ($vmval)" 
    } else {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)" 
    }
}

#####################
# Test DCUI user
$value = $ESXcli.system.account.list.Invoke() | Where-Object { $_.UserID -eq 'dcui' } | Select-Object -ExpandProperty Shellaccess
if ($value -eq 'false') {
    Write-Output "$name`:   [PASS] DCUI user has shell access deactivated ($value)" 
} else {
    Write-Output "$name`:   [FAIL] DCUI user has shell access enabled ($value)" 
}

#####################
# Test Entropy Sources
$value1 = $ESXcli.system.settings.kernel.list.Invoke() | Where-Object {$_.Name -eq "disableHwrng"} | Select-Object -ExpandProperty Configured
$value2 = $ESXcli.system.settings.kernel.list.Invoke() | Where-Object {$_.Name -eq "entropySources"} | Select-Object -ExpandProperty Configured

if ($value1 -eq 'FALSE' -and $value2 -eq '0') {
    Write-Output "$name`:   [PASS] Entropy sources configured correctly ($value1, $value2)" 
} else {
    Write-Output "$name`:   [FAIL] Entropy sources not configured correctly ($value1, $value2)" 
}

#####################
# Test Host Secure Boot Enforcement
$value = $ESXcli.system.settings.encryption.get.Invoke() | Select-Object -ExpandProperty RequireSecureBoot
if ($value -eq 'true') {
    Write-Output "$name`:   [PASS] Secure Boot enforcement is enabled ($value)" 
} else {
    Write-Output "$name`:   [FAIL] Secure Boot enforcement is not enabled ($value)" 
}

#####################
# Test TPM Configuration Encryption
$value = $ESXcli.system.settings.encryption.get.Invoke() | Select-Object -ExpandProperty Mode
if ($value -eq 'TPM') {
    Write-Output "$name`:   [PASS] TPM configuration encryption is enabled ($value)" 
} else {
    Write-Output "$name`:   [FAIL] TPM configuration encryption is not enabled ($value)" 
}

#####################
# Test Software Acceptance Level (VMwareCertified, VMwareAccepted, PartnerSupported, CommunitySupported)
$value = $ESXcli.software.acceptance.get.Invoke()
if (($value -eq 'PartnerSupported') -or ($value -eq 'VMwareCertified') -or ($value -eq 'VMwareAccepted')) {
    Write-Output "$name`:   [PASS] Host Image Profile Acceptance Level is configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] Host Image Profile Acceptance Level is not configured correctly ($value)" 
}

#####################
# Test authentication configuration
$value = $obj | Get-VMHostAuthentication | Select-Object -ExpandProperty Domain
if ($null -eq $value) {
    Write-Output "$name`:   [PASS] Active Directory integration is configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] Active Directory integration is not configured correctly ($value)" 
}

#####################
# Test the menagerie of SSH configuration settings
$value = $ESXcli.system.security.fips140.ssh.get.Invoke() | Select-Object -ExpandProperty Enabled
if ($value -eq 'true') {
    Write-Output "$name`:   [PASS] SSH has FIPS mode enabled ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH does not have FIPS mode enabled ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'ciphers'} | Select-Object -ExpandProperty Value
if ($value -eq 'aes256-gcm@openssh.com,aes128-gcm@openssh.com,aes256-ctr,aes192-ctr,aes128-ctr') {
    Write-Output "$name`:   [PASS] SSH ciphers configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH ciphers not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'gatewayports'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH gatewayports configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH gatewayports not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'hostbasedauthentication'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH hostbasedauthentication configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH hostbasedauthentication not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'clientalivecountmax'} | Select-Object -ExpandProperty Value
if ($value -eq '3') {
    Write-Output "$name`:   [PASS] SSH clientalivecountmax configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH clientalivecountmax not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'clientaliveinterval'} | Select-Object -ExpandProperty Value
if ($value -eq '200') {
    Write-Output "$name`:   [PASS] SSH clientaliveinterval configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH clientaliveinterval not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'banner'} | Select-Object -ExpandProperty Value
if ($value -eq '/etc/issue') {
    Write-Output "$name`:   [PASS] SSH banner configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH banner not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'ignorerhosts'} | Select-Object -ExpandProperty Value
if ($value -eq 'yes') {
    Write-Output "$name`:   [PASS] SSH ignorerhosts configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH ignorerhosts not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'allowstreamlocalforwarding'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH allowstreamlocalforwarding configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH allowstreamlocalforwarding not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'allowtcpforwarding'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH allowtcpforwarding configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH allowtcpforwarding not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'permittunnel'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH permittunnel configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH permittunnel not configured correctly ($value)" 
}

$value = $ESXcli.system.ssh.server.config.list.invoke() | Where-Object {$_.Key -eq 'permituserenvironment'} | Select-Object -ExpandProperty Value
if ($value -eq 'no') {
    Write-Output "$name`:   [PASS] SSH permituserenvironment configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] SSH permituserenvironment not configured correctly ($value)" 
}

#####################
# Test ESXi services
$services_should_be_false = "sfcbd-watchdog", "TSM", "slpd", "snmpd", "TSM-SSH"

foreach ($service in $services_should_be_false) {
    $value = $obj | Get-VMHostService | Where-Object {$_.Key -eq $service} | Select-Object -ExpandProperty Running
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] $service is not running ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] $service is running ($value)" 
    }

    $value = $obj | Get-VMHostService | Where-Object {$_.Key -eq $service} | Select-Object -ExpandProperty Policy
    if ($value -eq 'off') {
        Write-Output "$name`:   [PASS] $service is not configured to start ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] $service is configured to start ($value)" 
    }
}

$services_should_be_true = "ntpd"

foreach ($service in $services_should_be_true) {
    $value = $obj | Get-VMHostService | Where-Object {$_.Key -eq $service} | Select-Object -ExpandProperty Running
    if ($value -eq $true) {
        Write-Output "$name`:   [PASS] $service is not running ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] $service is running ($value)" 
    }

    $value = $obj | Get-VMHostService | Where-Object {$_.Key -eq $service} | Select-Object -ExpandProperty Policy
    if ($value -eq 'on') {
        Write-Output "$name`:   [PASS] $service is not configured to start ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] $service is configured to start ($value)" 
    }
}

#####################
# Test NTP configurations
$value = $obj | Get-VMHostNtpServer
if ($null -eq $value) {
    Write-Output "$name`:   [FAIL] NTP client not configured ($value)" 
} else {
    Write-Output "$name`:   [PASS] NTP client configured ($value)" 
}

#####################
# Test lockdown mode
$value = (Get-View ($view).ConfigManager.HostAccessManager).QueryLockdownExceptions()
if ($null -eq $value) {
    Write-Output "$name`:   [PASS] Lockdown Mode exception users configured correctly ($value)" 
} else {
    Write-Output "$name`:   [FAIL] Lockdown Mode exception users not configured correctly ($value)" 
}

$value = (Get-View ($view).ConfigManager.HostAccessManager).LockdownMode
if ($value -eq 'lockdownDisabled') {
    Write-Output "$name`:   [FAIL] Lockdown Mode is not configured correctly ($value)" 
} else {
    Write-Output "$name`:   [PASS] Lockdown Mode is configured correctly ($value)" 
}

#####################
# Test Standard Switches
$switches = Get-VirtualSwitch -VMHost $obj -Standard

foreach ($switch in $switches) {
    $value = $switch | Get-SecurityPolicy | Select-Object -ExpandProperty AllowPromiscuous
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Standard switch `'$switch`' is not configured to allow promiscuous mode ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Standard switch `'$switch`' is configured to allow promiscuous mode ($value)" 
    }

    $value = $switch | Get-SecurityPolicy | Select-Object -ExpandProperty MacChanges
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Standard switch `'$switch`' is not configured to allow MAC address changes ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Standard switch `'$switch`' is configured to allow MAC address changes ($value)" 
    }

    $value = $switch | Get-SecurityPolicy | Select-Object -ExpandProperty ForgedTransmits
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Standard switch `'$switch`' is not configured to allow forged transmits ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Standard switch `'$switch`' is configured to allow forged transmits ($value)" 
    }

    $portgroups = Get-VirtualPortGroup -VirtualSwitch $switch
    foreach ($portgroup in $portgroups) {
        $value = $portgroup | Get-SecurityPolicy | Select-Object -ExpandProperty AllowPromiscuous
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Standard portgroup `'$portgroup`' is not configured to allow promiscuous mode ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Standard portgroup `'$portgroup`' is configured to allow promiscuous mode ($value)" 
        }
    
        $value = $portgroup | Get-SecurityPolicy | Select-Object -ExpandProperty MacChanges
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Standard portgroup `'$portgroup`' is not configured to allow MAC address changes ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Standard portgroup `'$portgroup`' is configured to allow MAC address changes ($value)" 
        }
    
        $value = $portgroup | Get-SecurityPolicy | Select-Object -ExpandProperty ForgedTransmits
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Standard portgroup `'$portgroup`' is not configured to allow forged transmits ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Standard portgroup `'$portgroup`' is configured to allow forged transmits ($value)" 
        }

        $value = $portgroup | Select-Object -ExpandProperty VLanID
        if ($value -eq 4095) {
            Write-Output "$name`:   [FAIL] Standard portgroup `'$portgroup`' is configured to allow VLAN 4095 ($value)" 
        } else {
            Write-Output "$name`:   [PASS] Standard portgroup `'$portgroup`' is not configured to allow VLAN 4095 ($value)" 
        }
       
        if (($value -eq 1) -or ($null -eq $value)) {
            Write-Output "$name`:   [FAIL] Standard portgroup `'$portgroup`' may be configured to use a default VLAN and should be assessed ($value)" 
        } else {
            Write-Output "$name`:   [PASS] Standard portgroup `'$portgroup`' does not appear to be configured to use a default VLAN ($value)" 
        }
    }

}

$vmks = $obj | Get-VMHostNetworkAdapter -VMKernel
foreach ($vmk in $vmks) {
    $valueObj = $vmk | Select-Object ManagementTrafficEnabled,VMotionEnabled,FaultToleranceLoggingEnabled,
                                    VsanTrafficEnabled,ProvisioningEnabled,VSphereReplicationEnabled,
                                    VSphereReplicationNFCEnabled,VSphereBackupNFCEnabled
    $valueStr = ($valueObj.PSObject.Properties | ForEach-Object { "$($_.Name): $($_.Value)" }) -join ', '

    if ($vmk.ManagementTrafficEnabled) {
        if ($vmk.VMotionEnabled -or
            $vmk.FaultToleranceLoggingEnabled -or
            $vmk.VsanTrafficEnabled -or
            $vmk.ProvisioningEnabled -or
            $vmk.VSphereReplicationEnabled -or
            $vmk.VSphereReplicationNFCEnabled -or
            $vmk.VSphereBackupNFCEnabled) {
                Write-Output "$name`:   [FAIL] VMkernel NIC `'$vmk`' has management configured alongside other services and should be assessed ($valueStr)"
        } else {
            Write-Output "$name`:   [PASS] VMkernel NIC `'$vmk`' has only management configured ($valueStr)"
        }
    } else {
        Write-Output "$name`:   [PASS] VMkernel NIC `'$vmk`' is not configured for management ($valueStr)"
    }
}
