<#
    Script Name: VMware vSphere ESXi Host Security Settings Audit Utility
    Copyright (C) 2023 VMware, Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
#>

<#
    "And now that you don’t have to be perfect, you can be good." ~ John Steinbeck, East of Eden
#>

Param (
    # Omit all banners
    [Parameter(Mandatory=$false)]
    [switch]$NoBanners = $false,
    # Skip safety checks
    [Parameter(Mandatory=$false)]
    [switch]$NoSafetyChecks = $false
)

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if we have the required version of VMware.PowerCLI
    $installedVersion = (Get-InstalledModule -Name 'VMware.PowerCLI' -AllVersions -ErrorAction SilentlyContinue).Version | Sort-Object -Desc | Select-Object -First 1

    if ('13.0.0' -gt $installedVersion) {
        Write-Output "[ERROR] This script requires PowerCLI 13.0 or newer. Current version is $installedVersion" 
        Write-Output "[ERROR] Instructions for installation & upgrade can be found at https://developer.vmware.com/powercli" 
        Exit
    }

    #####################
    # Check to see if we are attached to a vCenter Server
    if (($global:DefaultVIServers.Count -lt 1) -or ($global:DefaultVIServers.Count -gt 1)) {
        Write-Output "[ERROR] Please connect to a single vCenter Server (use Connect-VIServer, Connect-CisServer, and Connect-SsoAdminServer) prior to running this script. Thank you." 
        Exit
    }
}

$name = $global:DefaultVIServers.Name

if ($NoBanners -eq $false) {
    
    #######################################################################################################
    # Header
    Write-Output "$name`:   [TEXT] VMware vSphere Security Configuration & Hardening Guide 802-20230930-01"
    Write-Output "$name`:   [TEXT] vCenter Security Settings Audit Utility"
    Write-Output "$name`:   [TEXT]"
}

# You want the date and time in the audit record.
$currentDateTime = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
Write-Output "$name`:   [INFO] Audit of $name started at $currentDateTime from $env:COMPUTERNAME by $env:USERNAME"
Write-Output "$name`:   [INFO]"

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if the host is the right version
    if ('8.0.0' -gt $global:DefaultVIServers.Version) {
        Write-Output "$name`:  [ERROR] This script is designed for vSphere 8. Please upgrade the connected vCenter." 
        Exit
    }
}

if ($NoBanners -eq $false) {
  
    #####################
    # Disclaimer
    Write-Output ("$name`:   [TEXT] See the included documentation for license information and disclaimer.") 
    Write-Output ("$name`:   [TEXT]")

}

#####################
# Tests for SSO 
$scg_adv = @{
    
    'vpxd.event.syslog.enabled' = $true
    'config.log.level' = 'info'
    'VirtualCenter.VimPasswordExpirationInDays' = 30

}

foreach ($param in $scg_adv.GetEnumerator() )
{
    $value = (Get-AdvancedSetting -Entity $global:DefaultVIServers.Name "$($param.Name)").Value

    if ($value -eq $($param.Value)) {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] $($param.Name) not configured correctly ($value)" 
    }
}

#####################
# Tests for things that should not be set a certain way
$scg_not = @{

    'etc.issue' = 'Platform Services Controller' # Just test to see if the stock message is there.

}

foreach ($param in $scg_not.GetEnumerator() )
{
    $value = (Get-AdvancedSetting -Entity $global:DefaultVIServers.Name -Name "$($param.Name)" | Select-Object -ExpandProperty Value)

    $singleline = $value -replace '\r?\n', ' '
    if ($value -match $($param.Value)) {
        Write-Output "$name`:   [FAIL] $($param.Name) not configured correctly ($singleline)" 
    } else {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($singleline)" 
    }
}

#####################
# Test SSO Lockout Configurations
$value = Get-SsoLockoutPolicy
if ($value.AutoUnlockIntervalSec -eq 0) {
    Write-Output "$name`:   [PASS] SSO AutoUnlockIntervalSec configured correctly ($($value.AutoUnlockIntervalSec))" 
} else {
    Write-Output "$name`:   [FAIL] SSO AutoUnlockIntervalSec not configured correctly ($($value.AutoUnlockIntervalSec))" 
}

if ($value.FailedAttemptIntervalSec -eq 900) {
    Write-Output "$name`:   [PASS] SSO FailedAttemptIntervalSec configured correctly ($($value.FailedAttemptIntervalSec))" 
} else {
    Write-Output "$name`:   [FAIL] SSO FailedAttemptIntervalSec not configured correctly ($($value.FailedAttemptIntervalSec))" 
}

if ($value.MaxFailedAttempts -eq 5) {
    Write-Output "$name`:   [PASS] SSO MaxFailedAttempts configured correctly ($($value.MaxFailedAttempts))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MaxFailedAttempts not configured correctly ($($value.MaxFailedAttempts))" 
}

#####################
# Test SSO Password Policy
$value = Get-SsoPasswordPolicy
if ($value.PasswordLifetimeDays -eq 9999) {
    Write-Output "$name`:   [PASS] SSO PasswordLifetimeDays configured correctly ($($value.PasswordLifetimeDays))" 
} else {
    Write-Output "$name`:   [FAIL] SSO PasswordLifetimeDays not configured correctly ($($value.PasswordLifetimeDays))" 
}

if ($value.ProhibitedPreviousPasswordsCount -eq 5) {
    Write-Output "$name`:   [PASS] SSO ProhibitedPreviousPasswordsCount configured correctly ($($value.ProhibitedPreviousPasswordsCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO ProhibitedPreviousPasswordsCount not configured correctly ($($value.ProhibitedPreviousPasswordsCount))" 
}

if ($value.MinLength -ge 15) {
    Write-Output "$name`:   [PASS] SSO MinLength configured correctly ($($value.MinLength))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinLength not configured correctly ($($value.MinLength))" 
}

if ($value.MaxLength -eq 64) {
    Write-Output "$name`:   [PASS] SSO MaxLength configured correctly ($($value.MaxLength))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MaxLength not configured correctly ($($value.MaxLength))" 
}

if ($value.MinNumericCount -eq 1) {
    Write-Output "$name`:   [PASS] SSO MinNumericCount configured correctly ($($value.MinNumericCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinNumericCount not configured correctly ($($value.MinNumericCount))" 
}

if ($value.MinSpecialCharCount -eq 1) {
    Write-Output "$name`:   [PASS] SSO MinSpecialCharCount configured correctly ($($value.MinSpecialCharCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinSpecialCharCount not configured correctly ($($value.MinSpecialCharCount))" 
}

if ($value.MaxIdenticalAdjacentCharacters -eq 3) {
    Write-Output "$name`:   [PASS] SSO MaxIdenticalAdjacentCharacters configured correctly ($($value.MaxIdenticalAdjacentCharacters))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MaxIdenticalAdjacentCharacters not configured correctly ($($value.MaxIdenticalAdjacentCharacters))" 
}

if ($value.MinAlphabeticCount -eq 2) {
    Write-Output "$name`:   [PASS] SSO MinAlphabeticCount configured correctly ($($value.MinAlphabeticCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinAlphabeticCount not configured correctly ($($value.MinAlphabeticCount))" 
}

if ($value.MinUppercaseCount -eq 1) {
    Write-Output "$name`:   [PASS] SSO MinUppercaseCount configured correctly ($($value.MinUppercaseCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinUppercaseCount not configured correctly ($($value.MinUppercaseCount))" 
}

if ($value.MinLowercaseCount -eq 1) {
    Write-Output "$name`:   [PASS] SSO MinLowercaseCount configured correctly ($($value.MinLowercaseCount))" 
} else {
    Write-Output "$name`:   [FAIL] SSO MinLowercaseCount not configured correctly ($($value.MinLowercaseCount))" 
}

#####################
# Test Distributed Switches
$switches = Get-VDSwitch

foreach ($switch in $switches) {
    $value = $switch | Get-VDSecurityPolicy | Select-Object -ExpandProperty AllowPromiscuous
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' is not configured to allow promiscuous mode ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' is configured to allow promiscuous mode ($value)" 
    }

    $value = $switch | Get-VDSecurityPolicy | Select-Object -ExpandProperty MacChanges
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' is not configured to allow MAC address changes ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' is configured to allow MAC address changes ($value)" 
    }

    $value = $switch | Get-VDSecurityPolicy | Select-Object -ExpandProperty ForgedTransmits
    if ($value -eq $false) {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' is not configured to allow forged transmits ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' is configured to allow forged transmits ($value)" 
    }

    $value = $switch.ExtensionData.Config.LinkDiscoveryProtocolConfig | Select-Object -ExpandProperty Operation
    if ($value -eq "none") {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' link discovery is configured correctly ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' link discovery is not configured correctly ($value)" 
    }

    $value = $switch.ExtensionData.Config.IpfixConfig | Select-Object -ExpandProperty CollectorIpAddress
    if (($value -eq "") -or ($null -eq $value)) {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' NetFlow is configured correctly ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' NetFlow is configured with a collector ($value)" 
    }

    $value = $switch.ExtensionData.Config | Select-Object -ExpandProperty VspanSession
    if ($null -eq $value) {
        Write-Output "$name`:   [PASS] Distributed switch `'$switch`' port mirroring is inactive ($value)" 
    } else {
        Write-Output "$name`:   [FAIL] Distributed switch `'$switch`' port mirroring is not active ($value)" 
    }

    $portgroups = Get-VDPortgroup -VDSwitch $switch | Where-Object {$_.ExtensionData.Config.Uplink -ne "True"}
    foreach ($portgroup in $portgroups) {
        $value = $portgroup | Get-VDSecurityPolicy | Select-Object -ExpandProperty AllowPromiscuous
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' is not configured to allow promiscuous mode ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' is configured to allow promiscuous mode ($value)" 
        }
    
        $value = $portgroup | Get-VDSecurityPolicy | Select-Object -ExpandProperty MacChanges
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' is not configured to allow MAC address changes ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' is configured to allow MAC address changes ($value)" 
        }
    
        $value = $portgroup | Get-VDSecurityPolicy | Select-Object -ExpandProperty ForgedTransmits
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' is not configured to allow forged transmits ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' is configured to allow forged transmits ($value)" 
        }

        $value = $portgroup | Select-Object -ExpandProperty VlanConfiguration
        if ($value -eq 4095) {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' is configured to allow VLAN 4095 ($value)" 
        } else {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' is not configured to allow VLAN 4095 ($value)" 
        }
       
        if (($value -eq 1) -or ($null -eq $value)) {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' may be configured to use a default VLAN and should be assessed ($value)" 
        } else {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' does not appear to be configured to use a default VLAN ($value)" 
        }

        $value = $portgroup.ExtensionData.Config.Policy | Select-Object -ExpandProperty PortConfigResetAtDisconnect
        if ($value -eq $true) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' is configured to reset port configuration on disconnect ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' is not configured to reset port configuration on disconnect ($value)" 
        }

        $value = $portgroup.ExtensionData.Config.DefaultPortConfig.IpfixEnabled | Select-Object -ExpandProperty Value
        if ($value -eq $false) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' NetFlow is configured correctly ($value)" 
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' NetFlow is not configured correctly ($value)" 
        }

        # Test port group policies
        $valueObj = $portgroup.ExtensionData.Config.Policy | Select-Object VlanOverrideAllowed, UplinkTeamingOverrideAllowed, 
                    SecurityPolicyOverrideAllowed, MacManagementOverrideAllowed, BlockOverrideAllowed, ShapingOverrideAllowed, IpfixOverrideAllowed,
                    VendorConfigOverrideAllowed, LivePortMovingAllowed, NetworkResourcePoolOverrideAllowed, TrafficFilterOverrideAllowed
        $valueStr = ($valueObj.PSObject.Properties | ForEach-Object { "$($_.Name): $($_.Value)" }) -join ', '

        if (
            $valueObj.VlanOverrideAllowed -eq $false -and
            $valueObj.UplinkTeamingOverrideAllowed -eq $false -and
            $valueObj.SecurityPolicyOverrideAllowed -eq $false -and
            $valueObj.MacManagementOverrideAllowed -eq $false -and
            $valueObj.BlockOverrideAllowed -eq $true -and
            $valueObj.ShapingOverrideAllowed -eq $false -and
            $valueObj.IpfixOverrideAllowed -eq $false -and
            $valueObj.VendorConfigOverrideAllowed -eq $false -and
            $valueObj.LivePortMovingAllowed -eq $false -and
            $valueObj.NetworkResourcePoolOverrideAllowed -eq $false -and
            $valueObj.TrafficFilterOverrideAllowed -eq $false
        ) {
            Write-Output "$name`:   [PASS] Distributed portgroup `'$portgroup`' policies are configured correctly ($valueStr)"
        } else {
            Write-Output "$name`:   [FAIL] Distributed portgroup `'$portgroup`' policies are not configured correctly ($valueStr)"
        }
    }

    #####################
    # Test VCSA Settings for SSH
    $value = (Get-CisService -Name "com.vmware.appliance.access.ssh").get()
    if ($value -eq $true) {
        Write-Output "$name`:   [FAIL] vCenter Server Appliance has SSH enabled ($value)" 
    } else {
        Write-Output "$name`:   [PASS] vCenter Server Appliance does not have SSH enabled ($value)" 
    }

    #####################
    # Test VCSA Settings for password policies
    $value = (Get-CisService -Name "com.vmware.appliance.local_accounts.policy").get() | Select-Object -ExpandProperty max_days
    if ($value -ne 9999) {
        Write-Output "$name`:   [FAIL] vCenter Server Appliance local accounts max_days not configured correctly ($value)" 
    } else {
        Write-Output "$name`:   [PASS] vCenter Server Appliance local accounts max_days configured correctly ($value)" 
    }

    #####################
    # Test VCSA Settings for log forwarding
    $value = (Get-CisService -Name "com.vmware.appliance.logging.forwarding").get()
    if ($value.Count -eq 0) {
        Write-Output "$name`:   [FAIL] vCenter Server Appliance not configured to forward logs ($($value.Hostname))" 
    } else {
        Write-Output "$name`:   [PASS] vCenter Server Appliance configured to forward logs ($($value.Hostname))" 
    }

    #####################
    # Test VCSA Settings for NTP
    $value = (Get-CisService -Name "com.vmware.appliance.timesync").get()
    if ($value -ne "NTP") {
        Write-Output "$name`:   [FAIL] vCenter Server Appliance NTP not configured ($value)" 
    } else {
        Write-Output "$name`:   [PASS] vCenter Server Appliance NTP configured correctly ($value)" 
    }

    $value = (Get-CisService -Name "com.vmware.appliance.ntp").get()
    if ($null -eq $value) {
        Write-Output "$name`:   [FAIL] vCenter Server Appliance NTP does not have servers defined ($value)" 
    } else {
        Write-Output "$name`:   [PASS] vCenter Server Appliance NTP has servers defined ($value)" 
    }


}

