<#
    Script Name: VMware vSphere Virtual Machine Security Settings Audit Utility
    Copyright (C) 2023 VMware, Inc. All rights reserved.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
#>

<#
    "And now that you don’t have to be perfect, you can be good." ~ John Steinbeck, East of Eden
#>

Param (
    # Virtual Machine Name
    [Parameter(Mandatory=$true)]
    [string]$Name,
    # Omit all banners
    [Parameter(Mandatory=$false)]
    [switch]$NoBanners = $false,
    # Skip safety checks
    [Parameter(Mandatory=$false)]
    [switch]$NoSafetyChecks = $false,
    # Skip safety checks except for appliances
    [Parameter(Mandatory=$false)]
    [switch]$NoSafetyChecksExceptAppliances = $false
)

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if we have the required version of VMware.PowerCLI
    $installedVersion = (Get-InstalledModule -Name 'VMware.PowerCLI' -AllVersions -ErrorAction SilentlyContinue).Version | Sort-Object -Desc | Select-Object -First 1

    if ('13.0.0' -gt $installedVersion) {
        Write-Output "[ERROR] This script requires PowerCLI 13.0 or newer. Current version is $installedVersion" 
        Write-Output "[ERROR] Instructions for installation & upgrade can be found at https://developer.vmware.com/powercli" 
        Exit
    }

    #####################
    # Check to see if we are attached to a vCenter Server
    if ($global:DefaultVIServers.Count -lt 1) {
        Write-Output "[ERROR] Please connect to a vCenter Server (use Connect-VIServer) prior to running this script. Thank you." 
        Exit
    }
}

if ($NoBanners -eq $false) {
    
    #######################################################################################################
    # Header
    Write-Output "$name`:   [TEXT] VMware vSphere Security Configuration & Hardening Guide 802-20230930-01"
    Write-Output "$name`:   [TEXT] Virtual Machine Security Settings Audit Utility"
    Write-Output "$name`:   [TEXT]"
}

# You want the date and time in the audit record.
$currentDateTime = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
Write-Output "$name`:   [INFO] Audit of $name started at $currentDateTime from $env:COMPUTERNAME by $env:USERNAME"
Write-Output "$name`:   [INFO]"

#####################
# Read the VM into objects and views once to save time & resources
$obj = Get-VM $name -ErrorAction Stop
$view = Get-View -VIObject $obj
$hosts = Get-VMHost

if ($NoSafetyChecks -eq $false) {
    #####################
    # Check to see if the host is the right version
    foreach ($hostVersion in $hosts.Version) {
        if ('8.0.0' -gt $hostVersion) {
            Write-Output "$name`:  [ERROR] This script is designed for vSphere 8. Please upgrade all connected hosts." 
            Exit
        }
    }
}

if ($NoBanners -eq $false) {
  
    #####################
    # Disclaimer
    Write-Output ("$name`:   [TEXT] See the included documentation for license information and disclaimer.") 
    Write-Output ("$name`:   [TEXT]")

}

if (($NoSafetyChecks -eq $false) -or ($NoSafetyChecksExceptAppliances -eq $true)) {
    #####################
    # Is this a VMware appliance?
    $flag = $false
    if ($obj | Select-Object -ExpandProperty Notes | Select-String -Pattern "VMware" -AllMatches) { $flag = $true}
    if ($obj | Select-Object -ExpandProperty Notes | Select-String -Pattern "vSphere Cluster Service" -AllMatches) { $flag = $true}

    if ($flag) {
        Write-Output "$name`:  [ERROR] VM may be a vSphere appliance. Altering VMware virtual appliances is not supported." 
        Exit
    }
}

#####################
# Test for Secure Boot
$value = $obj.ExtensionData.Config.BootOptions.EfiSecureBootEnabled
if ($value -eq $true) {
    Write-Output "$name`:   [PASS] Secure Boot configured ($value)" 
} else {
    Write-Output "$name`:   [FAIL] VM does not have Secure Boot configured ($value)" 
}

#####################
# Test for VM Hardware version
switch ($view.Config.Version) {
    'vmx-19' { Write-Output "$name`:   [PASS] VM is VM Hardware 19 (vSphere 7) ($value)"  }
    'vmx-20' { Write-Output "$name`:   [PASS] VM is VM Hardware 20 (vSphere 8) ($value)"  }
    'vmx-21' { Write-Output "$name`:   [PASS] VM is VM Hardware 21 (vSphere 8) ($value)"  }
    Default { Write-Output "$name`:   [FAIL] VM Hardware version should be version 19 or later ($value)"  }
} 

#####################
# Tests for advanced parameters
#
# Parameter guidelines for configurations requiring boolean values:
#
# T-OR-NP = The setting should be configured as TRUE or not present, where the default of TRUE will take over.
# F-OR-NP = The setting should be configured as FALSE or not present, where the default of FALSE will take over.
# TRUE = The setting should be configured as TRUE.
# FALSE = The setting should be configured as FALSE.
# NP = The setting should not be configured at all.
#
$scg_bool = @{
    
    'isolation.tools.copy.disable' = 'T-OR-NP'
    'isolation.tools.paste.disable' = 'T-OR-NP'
    'isolation.tools.diskShrink.disable' = 'T-OR-NP'
    'isolation.tools.diskWiper.disable' = 'T-OR-NP'
    'mks.enable3d' = 'F-OR-NP'
    'tools.guestlib.enableHostInfo' = 'F-OR-NP'
    'tools.guest.desktop.autolock' = 'T-OR-NP'
    'isolation.device.connectable.disable' = 'T-OR-NP'
    'isolation.tools.dnd.disable' = 'T-OR-NP'
    'sched.mem.pshare.salt' = 'NP'
    
}    

$scg_num_nondefault = @{
    
    'RemoteDisplay.maxConnections' = 1

}

$scg_num_default = @{
    
    'tools.setInfo.sizeLimit' = 1048576
    'log.keepOld' = 10
    'log.rotateSize' = 2048000

}

foreach ($param in $scg_bool.GetEnumerator() )
{
    $vmval = (Get-AdvancedSetting -Entity $obj "$($param.Name)").Value

    switch ($($param.Value)) {
        'T-OR-NP' { 
            switch ($vmval) {
                'TRUE' {  Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)"  }
                'FALSE' { Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)"  }
                '' {      Write-Output "$name`:   [PASS] $($param.Name) not configured and is using secure defaults ($vmval)"  }
                Default { Write-Output "$name`:   [FAIL] $($param.Name) configured to something unexpected ($vmval)"  }
            }
        }
        'F-OR-NP' { 
            switch ($vmval) {
                'TRUE' {  Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)"  }
                'FALSE' { Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)"  }
                '' {      Write-Output "$name`:   [PASS] $($param.Name) not configured and is using secure defaults ($vmval)" }
                Default { Write-Output "$name`:   [FAIL] $($param.Name) configured to something unexpected ($vmval)"  }
            }
        }
        'TRUE' { 
            switch ($vmval) {
                'TRUE' {  Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)"  }
                'FALSE' { Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)"  }
                '' {      Write-Output "$name`:   [FAIL] $($param.Name) not configured ($vmval)"  }
                Default { Write-Output "$name`:   [FAIL] $($param.Name) configured to something unexpected ($vmval)"  }
            }
        }
        'FALSE' { 
            switch ($vmval) {
                'TRUE' {  Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)"  }
                'FALSE' { Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)"  }
                '' {      Write-Output "$name`:   [FAIL] $($param.Name) not configured ($vmval)"  }
                Default { Write-Output "$name`:   [FAIL] $($param.Name) configured to something unexpected ($vmval)"  }
            }
        }
        'NP' { 
            if ($vmval) {
                Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)." 
            } else {
                Write-Output "$name`:   [PASS] $($param.Name) not configured ($vmval)." 
            }
            }
        Default { Write-Output "$name`:   [FAIL] $($param.Name) configured to something unexpected ($vmval)"  }
    }

}

foreach ($param in $scg_num_nondefault.GetEnumerator() )
{
    $vmval = (Get-AdvancedSetting -Entity $obj "$($param.Name)").Value

    if ($vmval -eq $($param.Value)) {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)" 
    } else {
        Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)." 
    }
}

foreach ($param in $scg_num_default.GetEnumerator() )
{
    $vmval = (Get-AdvancedSetting -Entity $obj "$($param.Name)").Value

    if ($vmval -eq $($param.Value)) {
        Write-Output "$name`:   [PASS] $($param.Name) configured correctly ($vmval)" 
    } elseif (!$vmval) {
        Write-Output "$name`:   [PASS] $($param.Name) not configured ($vmval)" 
    } else {
        Write-Output "$name`:   [FAIL] $($param.Name) configured incorrectly ($vmval)" 
    }
}

#####################
# Test for vMotion encryption
$value = $obj.ExtensionData.Config.MigrateEncryption
switch($value) {
    'required' { Write-Output "$name`:   [PASS] Encrypted vMotion configured correctly ($value)"  }
    'opportunistic' { Write-Output "$name`:   [FAIL] Encrypted vMotion defaults configured ($value)."  }
    Default { Write-Output "$name`:   [FAIL] Encrypted vMotion not configured ($value)"  }
}

#####################
# Test for Fault Tolerance encryption
$value = $obj.ExtensionData.Config.FtEncryptionMode
switch($value) {
    'ftEncryptionRequired' { Write-Output "$name`:   [PASS] Encrypted Fault Tolerance configured correctly ($value)"  }
    'ftEncryptionOpportunistic' { Write-Output "$name`:   [FAIL] Encrypted Fault Tolerance defaults configured ($value)"  }
    Default { Write-Output "$name`:   [FAIL] Encrypted Fault Tolerance not configured ($value)"  }
}

#####################
# Test for VM logging
$value = $obj.ExtensionData.Config.Flags.EnableLogging
switch($value) {
    'True' { Write-Output "$name`:   [PASS] Diagnostic logging configured correctly ($value)"  }
    Default { Write-Output "$name`:   [FAIL] Diagnostic logging not configured ($value)"  }
}

#####################
# Test for dvFilters
$value = Get-AdvancedSetting -Entity $obj 'ethernet*.filter*.name*'
if ($NULL -eq $value) {
    Write-Output "$name`:   [PASS] dvFilters not configured ($value)" 
} else {
    Write-Output "$name`:   [FAIL] dvFilters configured, verify they are legitimate ($value)" 
}

#####################
# Test for passthrough devices
$value = $obj | Get-PassthroughDevice
if ($NULL -eq $value) {
    Write-Output "$name`:   [PASS] Passthrough hardware devices not configured ($value)" 
} else {
    Write-Output "$name`:   [FAIL] Passthrough hardware configured ($value)" 
}

#####################
# Test for unnecessary devices
$UnnecessaryHardware = "VirtualUSBController|VirtualUSBXHCIController|VirtualParallelPort|VirtualFloppy|VirtualSerialPort|VirtualHdAudioCard|VirtualAHCIController|VirtualEnsoniq1371|VirtualCdrom"

$view.Config.Hardware.Device | Where-Object {$_.GetType().Name -match $UnnecessaryHardware} | Foreach-Object {
    $devname = $_.GetType().Name
    Write-Output "$name`:   [FAIL] $devname device present, evaluate and remove if not necessary." 
}
